import Cocoa
import objc


class PathDemoController(Cocoa.NSObject):
    button = objc.ivar()
    popup = objc.ivar()
    window = objc.ivar()
    demoView = objc.ivar()

    def awakeFromNib(self):
        #  Add the title of your new demo to the END of this array.

        titles = ["Rectangles", "Circles", "Bezier Paths", "Circle Clipping"]

        self.popup.removeAllItems()

        for t in titles:
            self.popup.addItemWithTitle_(t)

    @objc.IBAction
    def runAgain_(self, sender):
        self.select_(self)

    @objc.IBAction
    def select_(self, sender):
        self.demoView.setDemoNumber_(self.popup.indexOfSelectedItem())
        self.demoView.setNeedsDisplay_(True)

    def applicationShouldTerminateAfterLastWindowClosed_(self, application):
        return True



"""
This module defines a number of context managers. These are meant to be used
in the context of the with statement (introduced in Python 2.5).
"""
__all__ = ("CGSavedGState", "CGTransparencyLayer", "CGContextPage")
import Quartz.CoreGraphics as CG


class CGSavedGState(object):
    """
    Context manager for saving and restoring the graphics state.

    Usage::

        with  CGSavedGState(context):
            statement

    This is equivalent to:
        CGContextSaveGState(context)
        try:
            statement

        finally:
            CGContextRestoreGState(context)
    """

    def __init__(self, context):
        self.context = context

    def __enter__(self):
        CG.CGContextSaveGState(self.context)
        return self

    def __exit__(self, exc_type, exc_value, exc_tp):
        CG.CGContextRestoreGState(self.context)
        return False


class CGTransparencyLayer(object):
    """
    Context manager for working in a transparancylayer.

    Usage::

        with CGTransparencyLayer(context, info [, rect]):
            statement

    This is equivalent to:
        CGContextBeginTransparencyLayer(context, info)
        try:
            statement

        finally:
            CGContextEndTransparencyLayer(context)
    """

    def __init__(self, context, info, rect=None):
        self.context = context
        self.info = info
        self.rect = rect

    def __enter__(self):
        if self.rect is None:
            result = CG.CGContextBeginTransparencyLayer(self.context, self.info)
        else:
            result = CG.CGContextBeginTransparencyLayerWithRect(
                self.context, self.rect, self.info
            )
        return result

    def __exit__(self, exc_type, exc_value, exc_tp):
        CG.CGContextEndTransparencyLayer(self.context)
        return False


class CGContextPage(object):
    """
    Context manager for saving and restoring the graphics state.

    Usage::

        with CGContextPage(context) as mediaRect:
            statement

    This is equivalent to:
        mediaRect = CGContextBeginPage(context, None)
        try:
            statement

        finally:
            CGContextEndPage(context)
    """

    def __init__(self, context, mediaBox=None):
        self.context = context
        self.mediaBox = mediaBox

    def __enter__(self):
        mediaRect = CG.CGContextBeginPage(self.context, self.mediaBox)
        return mediaRect

    def __exit__(self, exc_type, exc_value, exc_tp):
        CG.CGContextEndPage(self.context)
        return False



import Quartz
import Utilities


def scaleShadowOffset(offset):
    shadowScaling = Utilities.getScalingFactor()
    # Adjust the shadow offset if scaling to export as bits. This is
    # equivalent to scaling base space by the scaling factor.
    if shadowScaling != 1.0:
        offset = Quartz.CGSizeApplyAffineTransform(
            offset, Quartz.CGAffineTransformMakeScale(shadowScaling, shadowScaling)
        )
    return offset


def createTrianglePath(context):
    Quartz.CGContextBeginPath(context)
    Quartz.CGContextMoveToPoint(context, 0, 0)
    Quartz.CGContextAddLineToPoint(context, 50, 0)
    Quartz.CGContextAddLineToPoint(context, 25, 50)
    Quartz.CGContextClosePath(context)


def drawSimpleShadow(context):
    r = Quartz.CGRectMake(20, 20, 100, 200)

    Quartz.CGContextTranslateCTM(context, 20, 300)

    # A blur of 0 is a hard edge blur.
    blur = 0
    # An offset where both components are negative casts a shadow to the
    # left and down from the object. The coordinate system for the offset
    # is base space, not current user space.
    offset = Quartz.CGSize(-7, -7)
    offset = scaleShadowOffset(offset)

    # Set the shadow in the context.
    Quartz.CGContextSetShadow(context, offset, blur)

    # Object 1.
    # Paint a rectangle.
    Quartz.CGContextFillRect(context, r)

    # Object 2.
    Quartz.CGContextTranslateCTM(context, 150, 0)
    # A blur of 3 is a soft blur more
    # appropriate for a shadow effect.
    blur = 3
    Quartz.CGContextSetShadow(context, offset, blur)

    # Fill an ellipse to the right of the rect.
    Quartz.CGContextBeginPath(context)
    Utilities.myCGContextAddEllipseInRect(context, r)
    Quartz.CGContextFillPath(context)

    # Object 3.
    Quartz.CGContextTranslateCTM(context, -130, -140)
    # Scale the coordinate system but the shadow is not affected. The offset
    # is in the base space of the context. Typically it looks best if the shapes
    # have a uniform shadow regardless of how the shapes were created, scaled,
    # rotated, or otherwise transformed.
    Quartz.CGContextScaleCTM(context, 2, 2)
    createTrianglePath(context)
    Quartz.CGContextSetStrokeColorWithColor(context, Utilities.getRGBOpaqueRedColor())

    Quartz.CGContextSetLineWidth(context, 5)
    # Stroking produces a shadow as well.
    Quartz.CGContextStrokePath(context)

    # Object 4.
    Quartz.CGContextTranslateCTM(context, 75, 0)
    createTrianglePath(context)
    # Cast the shadow to the left and up from
    # the shape painted.
    offset.width = -5
    offset.height = +7
    offset = scaleShadowOffset(offset)

    # The shadow can be colored. Create a CGColorRef
    # that represents a red color with opacity of 0.3333...
    shadowColor = Quartz.CGColorCreateCopyWithAlpha(
        Utilities.getRGBOpaqueRedColor(), 1.0 / 3.0
    )

    Quartz.CGContextSetShadowWithColor(context, offset, blur, shadowColor)
    Quartz.CGContextStrokePath(context)

    # Object 5. Three stroked circles.
    Quartz.CGContextTranslateCTM(context, -75, -65)
    # Set a black shadow offset at -7,-7.
    offset.width = -7
    offset.height = -7

    offset = scaleShadowOffset(offset)

    Quartz.CGContextSetShadow(context, offset, blur)
    # Draw a set of three circles side by side.
    Quartz.CGContextBeginPath(context)
    Quartz.CGContextSetLineWidth(context, 3)
    r = Quartz.CGRectMake(30, 20, 20, 20)
    Utilities.myCGContextAddEllipseInRect(context, r)
    r = Quartz.CGRectOffset(r, 20, 0)
    Utilities.myCGContextAddEllipseInRect(context, r)
    r = Quartz.CGRectOffset(r, 20, 0)
    Utilities.myCGContextAddEllipseInRect(context, r)
    Quartz.CGContextStrokePath(context)


def doShadowScaling(context):
    offset = Quartz.CGSize(-7, -7)
    blur = 3

    Quartz.CGContextTranslateCTM(context, 20, 220)
    Quartz.CGContextSetShadow(context, scaleShadowOffset(offset), blur)

    # Object 1
    # Draw a triangle filled with black and shadowed with black.
    createTrianglePath(context)
    Quartz.CGContextFillPath(context)

    # Object 2
    # Scaling without changing the shadow doesn't impact
    # the shadow offset or blur.
    t = Quartz.CGAffineTransformMakeScale(2, 2)
    Quartz.CGContextConcatCTM(context, t)
    Quartz.CGContextTranslateCTM(context, 40, 0)
    createTrianglePath(context)
    Quartz.CGContextFillPath(context)

    # Object 3
    # By transforming the offset you can transform the shadow.
    # This may be desirable if you are drawing a zoomed view.
    offset = Quartz.CGSizeApplyAffineTransform(offset, t)
    Quartz.CGContextSetShadow(context, scaleShadowOffset(offset), blur)
    Quartz.CGContextTranslateCTM(context, 70, 0)
    createTrianglePath(context)
    Quartz.CGContextFillPath(context)


def drawFillAndStrokeWithShadow(context):
    r = Quartz.CGRectMake(60, 60, 100, 100)
    offset = Quartz.CGSize(-7, -7)
    blur = 3

    # Set the shadow.
    Quartz.CGContextSetShadow(context, scaleShadowOffset(offset), blur)

    Quartz.CGContextSetFillColorWithColor(context, Utilities.getRGBOpaqueOrangeColor())

    # Draw the graphic on the left.
    Quartz.CGContextBeginPath(context)
    Utilities.myCGContextAddEllipseInRect(context, r)
    Quartz.CGContextDrawPath(context, Quartz.kCGPathFillStroke)

    # Draw the graphic on the right.
    r = Quartz.CGRectOffset(r, 125, 0)
    # Begin the transparency layer.
    Quartz.CGContextBeginTransparencyLayer(context, None)
    if 1:
        Utilities.myCGContextAddEllipseInRect(context, r)
        Quartz.CGContextDrawPath(context, Quartz.kCGPathFillStroke)
    # End the transparency layer.
    Quartz.CGContextEndTransparencyLayer(context)


def drawColoredLogo(context):
    r = Quartz.CGRectMake(0, 0, 100, 100)
    Quartz.CGContextSaveGState(context)
    if 1:
        # Position the center of the rectangle on the left.
        Quartz.CGContextTranslateCTM(context, 140, 140)
        # Rotate so that the rectangles are rotated 45 degrees
        # about the current coordinate origin.
        Quartz.CGContextRotateCTM(context, Utilities.DEGREES_TO_RADIANS(45))
        # Translate so that the center of the rect is at the previous origin.
        Quartz.CGContextTranslateCTM(context, -r.size.width / 2, -r.size.height / 2)
        # Set the fill color to a purple color.
        Quartz.CGContextSetFillColorWithColor(
            context, Utilities.getRGBOpaquePurpleColor()
        )
        # Fill the first rectangle.
        Quartz.CGContextFillRect(context, r)
        # Position to draw the right-most rectangle.
        Quartz.CGContextTranslateCTM(context, 60, -60)
        # Set the fill color to a yellow color.
        Quartz.CGContextSetFillColorWithColor(
            context, Utilities.getRGBOpaqueYellowColor()
        )
        Quartz.CGContextFillRect(context, r)

        # Position for the center rectangle.
        Quartz.CGContextTranslateCTM(context, -30, +30)
        # Set the stroke color to an orange color.
        Quartz.CGContextSetStrokeColorWithColor(
            context, Utilities.getRGBOpaqueOrangeColor()
        )
        # Stroke the rectangle with a linewidth of 12.
        Quartz.CGContextStrokeRectWithWidth(context, r, 12)
    Quartz.CGContextRestoreGState(context)


def showComplexShadowIssues(context):
    offset = Quartz.CGSize(-6, -6)
    blur = 3

    # Set the shadow.
    Quartz.CGContextSetShadow(context, scaleShadowOffset(offset), blur)
    # Draw the colored logo.
    drawColoredLogo(context)


def showComplexShadow(context):
    offset = Quartz.CGSize(-6, -6)
    blur = 3

    # Set the shadow.
    Quartz.CGContextSetShadow(context, scaleShadowOffset(offset), blur)

    # Begin a transparency layer. A snapshot is made of the graphics state and
    # the shadow parameter is temporarily reset to no shadow, the blend mode
    # is set to Normal, and the global alpha parameter is set to 1.0.
    #
    # All drawing that occurs after CGContextBeginTransparencyLayer but before
    # CGContextEndTransparencyLayer is collected together and when
    # CGContextEndTransparencyLayer is called, Quartz composites the collected
    # drawing to the context, using the global alpha, blend mode, and shadow
    # that was in effect when CGContextBeginTransparencyLayer was called.

    Quartz.CGContextBeginTransparencyLayer(context, None)
    # Draw the colored logo.
    drawColoredLogo(context)

    # Ending the transparency layer causes all drawing in the transparency
    # layer to be composited with the global alpha, blend mode, and shadow
    # in effect at the time CGContextBeginTransparencyLayer was called.  The
    # graphics state is restored to that in effect when
    # CGContextBeginTransparencyLayer was called.

    # This restores the graphics state to that in effect
    # at the last call to CGContextBeginTransparencyLayer.
    Quartz.CGContextEndTransparencyLayer(context)


def doLayerCompositing(context):
    r = Quartz.CGRectMake(40, 50, 142, 180)
    # Object 1.
    Quartz.CGContextTranslateCTM(context, 20, 20)
    Quartz.CGContextSetFillColorWithColor(context, Utilities.getRGBOpaqueGreenColor())
    # Draw a green background.
    Quartz.CGContextFillRect(context, r)
    # Draw the colored logo.
    drawColoredLogo(context)

    # Object 2.
    Quartz.CGContextTranslateCTM(context, 300, 0)
    Quartz.CGContextSetFillColorWithColor(context, Utilities.getRGBOpaqueGreenColor())
    # Draw a green background.
    Quartz.CGContextFillRect(context, r)

    # Draw the rectangles with opacity 0.75.
    Quartz.CGContextSetAlpha(context, 0.75)

    drawColoredLogo(context)

    # Object 3.
    Quartz.CGContextTranslateCTM(context, 300, 0)
    # Set the alpha to 1.0 for drawing the background.
    Quartz.CGContextSetAlpha(context, 1.0)
    Quartz.CGContextSetFillColorWithColor(context, Utilities.getRGBOpaqueGreenColor())
    Quartz.CGContextFillRect(context, r)
    # Draw the rectangles with opacity 0.75.
    Quartz.CGContextSetAlpha(context, 0.75)
    # Begin a transparency layer. Drawing collected in
    # this transparency layer will be composited with an
    # alpha value of 0.75 when the transparency layer is ended.
    Quartz.CGContextBeginTransparencyLayer(context, None)
    if 1:
        # Draw the colored logo into the transparency layer.
        drawColoredLogo(context)

    # Ending the transparency layer causes the drawing
    # to then be composited with the global alpha value
    # in effect when CGContextBeginTransparencyLayer was called.
    Quartz.CGContextEndTransparencyLayer(context)


def shadowPDFDocument(context, url):
    pdfDoc = Quartz.CGPDFDocumentCreateWithURL(url)
    offset = Quartz.CGSize(-7, -7)

    if pdfDoc is None:
        print("Couldn't create PDF document reference!")
        return

    r = Quartz.CGPDFDocumentGetMediaBox(pdfDoc, 1)
    r.origin.x = 20
    r.origin.y = 20

    # Set the shadow.
    Quartz.CGContextSetShadow(context, scaleShadowOffset(offset), 3)

    # On Tiger and later, there is no need to use
    # a transparency layer to draw a PDF document as
    # a grouped object. On Panther, you can do so
    # by using a transparency layer. Drawing collected in
    # this transparency layer is drawn with the shadow
    # when the layer is ended.
    Quartz.CGContextBeginTransparencyLayer(context, None)
    if 1:
        Quartz.CGContextDrawPDFDocument(context, r, pdfDoc, 1)

    Quartz.CGContextEndTransparencyLayer(context)



import math

import Cocoa
import objc
import Quartz  # noqa: F401


class Circle(Cocoa.NSObject):
    radius = objc.ivar(type=objc._C_FLT)
    center = objc.ivar(type=Cocoa.NSPoint.__typestr__)
    color = objc.ivar()

    def bounds(self):
        return Cocoa.NSMakeRect(
            self.center.x - self.radius,
            self.center.y - self.radius,
            2 * self.radius,
            2 * self.radius,
        )

    def draw(self):
        context = Cocoa.NSGraphicsContext.currentContext().graphicsPort()

        self.color.set()
        Cocoa.CGContextSetGrayStrokeColor(context, 0, 1)
        Cocoa.CGContextSetLineWidth(context, 1.5)

        Cocoa.CGContextSaveGState(context)

        Cocoa.CGContextTranslateCTM(context, self.center.x, self.center.y)
        Cocoa.CGContextScaleCTM(context, self.radius, self.radius)
        Cocoa.CGContextMoveToPoint(context, 1, 0)
        Cocoa.CGContextAddArc(context, 0, 0, 1, 0, 2 * math.pi, False)
        Cocoa.CGContextClosePath(context)

        Cocoa.CGContextRestoreGState(context)
        Cocoa.CGContextDrawPath(context, Cocoa.kCGPathFill)



from __future__ import print_function

import sys

import Cocoa
import Quartz

DEBUG = 1


class MyConverterData(object):
    def __init__(self):
        self.doProgress = False
        self.abortConverter = False
        self.outStatusFile = sys.stdout
        self.converter = None


# Converter callbacks


def begin_document_callback(info):
    print("\nBegin Document\n", file=info.outStatusFile)


def end_document_callback(info, success):
    if success:
        success = "success"
    else:
        success = "failed"

    print("\nEnd Document: %s\n" % (success,), file=info.outStatusFile)


def begin_page_callback(info, pageno, page_info):
    print("\nBeginning page %d\n" % (pageno,), file=info.outStatusFile)
    if DEBUG:
        if page_info is not None:
            Cocoa.CFShow(page_info)


def end_page_callback(info, pageno, page_info):
    print("\nEnd page %d\n" % (pageno,), file=info.outStatusFile)
    if DEBUG:
        if page_info is not None:
            Cocoa.CFShow(page_info)


def progress_callback(info):
    if info.doProgress:
        info.outStatusFile.write(".")
        info.outStatusFile.flush()

    # Here would be a callout to code that
    # would conceivably return whether to abort
    # the conversion process.

    # UpdateStatus(converterDataP);

    if info.abortConverter:
        Quartz.CGPSConverterAbort(info.converter)
        print("ABORTED!", file=info.outStatusFile)


def message_callback(info, cfmessage):
    # Extract an ASCII version of the message. Typically
    # these messages are similar to those obtained from
    # any PostScript interpreter. Messages of the form
    # "%%[ Error: undefined; OffendingCommand: bummer ]%%"
    # are PostScript error messages and are the typical
    # reason a conversion job fails.

    print("\nMessage: %s" % (cfmessage), file=info.outStatusFile)


#  Given an input URL and a destination output URL, convert
#    an input PS or EPS file to an output PDF file. This conversion
#    can be time intensive and perhaps should be performed on
#    a secondary thread or by another process. */
def convertPStoPDF(inputPSURL, outPDFURL):
    provider = Quartz.CGDataProviderCreateWithURL(inputPSURL)
    consumer = Quartz.CGDataConsumerCreateWithURL(outPDFURL)

    if provider is None or consumer is None:
        if provider is None:
            print("Couldn't create provider")

        if consumer is None:
            print("Couldn't create consumer")

        return False

    # Setup the info data for the callbacks to
    # do progress reporting, set the initial state
    # of the abort flag to false and use stdout
    # as the file to write status and other information.
    myConverterData = MyConverterData()
    myConverterData.doProgress = True
    myConverterData.abortConverter = False
    myConverterData.outStatusFile = sys.stdout

    # Create a converter object with myConverterData as the
    # info parameter and our callbacks as the set of callbacks
    # to use for the conversion. There are no converter options
    # defined as of Tiger so the options dictionary passed is None.
    myConverterData.converter = Quartz.CGPSConverterCreate(
        myConverterData,
        (
            begin_document_callback,
            end_document_callback,
            begin_page_callback,
            end_page_callback,
            progress_callback,
            message_callback,
            None,  # no release_info_callback
        ),
        None,
    )

    if myConverterData.converter is None:
        print("Couldn't create converter object!")
        return False

    # There are no conversion options so the options
    # dictionary for the conversion is None.
    success = Quartz.CGPSConverterConvert(
        myConverterData.converter, provider, consumer, None
    )
    if not success:
        print("Conversion failed!")

    # There is no CGPSConverterRelease function. Since
    # a CGPSConverter object is a CF object, use CFRelease
    # instead.
    return success


def main(args=None):
    if args is None:
        args = sys.argv

    if len(args) != 3:
        print("Usage: %s inputfile outputfile." % (args[0],))
        return 0

    infile = args[1]
    outfile = args[2]

    if not isinstance(infile, bytes):
        infile = infile.encode("utf-8")

    if not isinstance(outfile, bytes):
        outfile = outfile.encode("utf-8")

    # Create the data provider and data consumer.
    inputURL = Cocoa.CFURLCreateFromFileSystemRepresentation(
        None, infile, len(infile), False
    )

    outputURL = Cocoa.CFURLCreateFromFileSystemRepresentation(
        None, outfile, len(outfile), False
    )

    if inputURL is not None and outputURL is not None:
        convertPStoPDF(inputURL, outputURL)

    return 0


if __name__ == "__main__":
    sys.exit(main())



import CGImageUtils
import Cocoa
import objc
import Quartz


class CGImageView(Cocoa.NSView):
    _image = objc.ivar()

    def setImage_(self, img):
        if img is not None and self._image is not img:
            self._image = img
            # Mark this view as needing to be redisplayed.
            self.setNeedsDisplay_(True)

    def image(self):
        return self._image

    def drawRect_(self, rect):
        # Obtain the current context
        ctx = Cocoa.NSGraphicsContext.currentContext().graphicsPort()

        # Draw the image in the context
        CGImageUtils.IIDrawImageTransformed(
            self._image,
            ctx,
            Quartz.CGRectMake(
                rect.origin.x, rect.origin.y, rect.size.width, rect.size.height
            ),
        )

        # Draw the view border, just a simple stroked rectangle
        Quartz.CGContextAddRect(
            ctx,
            Quartz.CGRectMake(
                rect.origin.x, rect.origin.y, rect.size.width, rect.size.height
            ),
        )
        Quartz.CGContextSetRGBStrokeColor(ctx, 1.0, 0.0, 0.0, 1.0)
        Quartz.CGContextStrokePath(ctx)



"""
Python mapping for the CoreVideo framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""
import sys

import CoreFoundation
import objc
from Quartz.CoreVideo import _metadata

sys.modules["Quartz.CoreVideo"] = mod = objc.ObjCLazyModule(
    "Quartz.CoreVideo",
    "com.apple.CoreVideo",
    objc.pathForFramework("/System/Library/Frameworks/CoreVideo.framework"),
    _metadata.__dict__,
    None,
    {
        "__doc__": __doc__,
        "__path__": __path__,
        "__loader__": globals().get("__loader__", None),
        "objc": objc,
    },
    (CoreFoundation,),
)


del sys.modules["Quartz.CoreVideo._metadata"]


def _load(mod):
    import Quartz

    Quartz.CoreVideo = mod
    import Quartz.CoreVideo._CVPixelBuffer as m

    for nm in dir(m):
        if nm.startswith("_"):
            continue
        setattr(mod, nm, getattr(m, nm))


_load(mod)



import AppDrawing
import Cocoa
import DataProvidersAndConsumers
import FrameworkUtilities
import Quartz


def createNewPDFRefFromPasteBoard():
    # Create a reference to the PDF data on the pasteboard.
    # The implementation of myCreatePDFDataFromPasteBoard depends
    # on the application framework you are using for your application.
    #
    # myCreatePDFDataFromPasteBoard creates a reference that is owned
    # by the calling application.
    pasteBoardData = FrameworkUtilities.myCreatePDFDataFromPasteBoard()

    if pasteBoardData is None:
        print("There is no PDF data on pasteboard!")
        return None

    # Create a data provider from the pasteboard data.
    dataProvider = Quartz.CGDataProviderCreateWithCFData(pasteBoardData)
    # Release the pasteboard data since the data provider retains
    # it and this code owns a reference but no longer requires it.
    del pasteBoardData

    if dataProvider is None:
        print("Couldn't create data provider.")
        return None

    pasteBoardPDFDocument = Quartz.CGPDFDocumentCreateWithProvider(dataProvider)
    # Release the data provider now that the code is done with it.
    del dataProvider

    if pasteBoardPDFDocument is None:
        print("Couldn't create PDF document from pasteboard data provider.")
        return None
    return pasteBoardPDFDocument


_pdfDoc = None


def getPasteBoardPDFDoc(reset):
    global _pdfDoc
    if reset:
        # Release any existing document.
        _pdfDoc = createNewPDFRefFromPasteBoard()
    else:
        # If there isn't already one, create it fresh.
        if _pdfDoc is None:
            _pdfDoc = createNewPDFRefFromPasteBoard()
    return _pdfDoc


def drawPasteBoardPDF(context):
    pdfDoc = getPasteBoardPDFDoc(False)  # Obtain the existing one.
    if pdfDoc is None:
        print("Quartz couldn't create CGPDFDocumentRef from pasteboard.")
        return

    # The media box is the bounding box of the PDF document.
    pdfRect = Quartz.CGPDFDocumentGetMediaBox(pdfDoc, 1)
    # page 1
    # Make the destination rect origin at the Quartz origin.
    pdfRect.origin.x = pdfRect.origin.y = 0.0
    Quartz.CGContextDrawPDFDocument(context, pdfRect, pdfDoc, 1)
    # page 1


def cfDataCreatePDFDocumentFromCommand(command):
    # Media rect for the drawing. In a real application this
    # should be the bounding rectangle of the graphics
    # that will be the PDF content.
    mediaRect = Quartz.CGRectMake(0, 0, 612, 792)

    # Create a dictionary to hold the optional information describing the PDF data.
    info_dict = {}

    # Add the creator and title information to the PDF content.
    info_dict[Quartz.kCGPDFContextTitle] = "Pasted From Sample Quartz Application"
    info_dict[Quartz.kCGPDFContextCreator] = "Sample Quartz Application"

    # Create a mutable CFData object with unlimited capacity.
    data = Cocoa.CFDataCreateMutable(None, 0)
    if data is None:
        print("Couldn't make CFData!")
        return None

    # Create the data consumer to capture the PDF data.
    consumer = DataProvidersAndConsumers.myCGDataConsumerCreateWithCFData(data)
    if consumer is None:
        print("Couldn't create data consumer!")
        return None

    pdfContext, mediaRect = Quartz.CGPDFContextCreate(consumer, None, info_dict)
    del consumer
    del info_dict

    if pdfContext is None:
        print("Couldn't create pdf context!")
        return None

    mediaRect = Quartz.CGContextBeginPage(pdfContext)
    if 1:
        Quartz.CGContextSaveGState(pdfContext)
        if 1:
            Quartz.CGContextClipToRect(pdfContext, mediaRect)
            AppDrawing.DispatchDrawing(pdfContext, command)
        Quartz.CGContextRestoreGState(pdfContext)
    Quartz.CGContextEndPage(pdfContext)

    return data


def MakePDFDocument(url, exportInfo):
    # Use this as the media box for the document.
    # In a real application this should be the bounding
    # rectangle of the graphics that will be the PDF content.
    mediaRect = Quartz.CGRectMake(0, 0, 612, 792)

    info = {
        # Add the title information for this document.
        Quartz.kCGPDFContextTitle: "BasicDrawing Sample Graphics",
        # Add the author information for this document. This is typically
        # the user creating the document.
        Quartz.kCGPDFContextAuthor: "David Gelphman and Bunny Laden",
        # The creator is the application creating the document.
        Quartz.kCGPDFContextCreator: "BasicDrawing Application",
    }

    if 0:
        # Before using the kCGPDFContextCropBox key, check to ensure that it
        # is available.
        if hasattr(Quartz, "kCFPDFContextCropBox"):
            # Prepare the crop box entry. Use this rectangle as the crop box for
            # this example.

            # XXX:fixme: need to encode as CFData!!!
            info[Quartz.kCGPDFContextCropBox] = Quartz.CGRectMake(100, 100, 200, 200)

    if url is not None:
        pdfContext = Quartz.CGPDFContextCreateWithURL(url, mediaRect, info)
        if pdfContext is not None:
            Quartz.CGContextBeginPage(pdfContext, mediaRect)
            if 1:
                Quartz.CGContextSaveGState(pdfContext)
                if 1:
                    Quartz.CGContextClipToRect(pdfContext, mediaRect)
                    AppDrawing.DispatchDrawing(pdfContext, exportInfo.command)
                Quartz.CGContextRestoreGState(pdfContext)
            Quartz.CGContextEndPage(pdfContext)
            del pdfContext
        else:
            print("Can't create PDF document!")



"""
Parse a PDF file and print some information about it.

Usage:
    python parse_page_contents.py inputfile ...
"""
from __future__ import print_function

import sys

import Cocoa
import objc
import Quartz


class MyDataScan(object):
    def __init__(self):
        self.numImagesWithColorThisPage = 0
        self.numImageMasksThisPage = 0
        self.numImagesMaskedWithMaskThisPage = 0
        self.numImagesMaskedWithColorsThisPage = 0


def printPageResults(outFile, myData, pageNum):
    if myData.numImagesWithColorThisPage:
        print(
            "Found %d images with intrinsic color on page %d."
            % (myData.numImagesWithColorThisPage, pageNum),
            file=outFile,
        )

    if myData.numImageMasksThisPage:
        print(
            "Found %d image masks on page %d."
            % (myData.numImageMasksThisPage, pageNum),
            file=outFile,
        )

    if myData.numImagesMaskedWithMaskThisPage:
        print(
            "Found %d images masked with masks on page %d."
            % (myData.numImagesMaskedWithMaskThisPage, pageNum),
            file=outFile,
        )

    if myData.numImagesMaskedWithColorsThisPage:
        print(
            "Found %d images masked with colors on page %d."
            % (myData.numImagesMaskedWithColorsThisPage, pageNum),
            file=outFile,
        )


def printDocResults(outFile, totPages, totImages):
    print("", file=outFile)
    print(
        "Summary: %d page document contains %d images." % (totPages, totImages),
        file=outFile,
    )
    print("", file=outFile)


def checkImageType(imageDict, myScanData):
    hasMaskKey, isMask = Quartz.CGPDFDictionaryGetBoolean(imageDict, "ImageMask", None)
    if not hasMaskKey:
        hasMaskKey, isMask = Quartz.CGPDFDictionaryGetBoolean(imageDict, "IM", None)

    if hasMaskKey and isMask:
        myScanData.numImageMasksThisPage += 1
        return

    # If image is masked with an alpha image it has an SMask entry.
    hasSMaskKey, pdf_object = Quartz.CGPDFDictionaryGetObject(imageDict, "SMask", None)
    if hasSMaskKey:
        # This object must be an XObject that is an image.
        # This code assumes the PDF is well formed in this regard.
        myScanData.numImagesMaskedWithMaskThisPage += 1
        return

    # If this image is masked with an image or with colors it has
    # a Mask entry.
    hasMask, pdf_object = Quartz.CGPDFDictionaryGetObject(imageDict, "Mask", None)
    if hasMask:
        # If the object is an XObject then the mask is an image.
        # If it is an array, the mask is an array of colors.
        object_type = Quartz.CGPDFObjectGetType(pdf_object)
        # Check if it is a stream type which it must be to be an XObject.
        if object_type == Quartz.kCGPDFObjectTypeStream:
            myScanData.numImagesMaskedWithMaskThisPage += 1
        elif object_type == Quartz.kCGPDFObjectTypeArray:
            myScanData.numImagesMaskedWithColorsThisPage += 1
        else:
            print("Mask entry in Image object is not well formed!")

        return

    # This image is not a mask, is not masked with another image or
    # color so it must be an image with intrinsic color with no mask.
    myScanData.numImagesWithColorThisPage += 1


#       The "Do" operator consumes one value off the stack, the name of
#       the object to execute. The name is a resource in the resource
#       dictionary of the page and the object corresponding to that name
#       is an XObject. The most common types of XObjects are either
#       Form objects or Image objects. This code only counts images.
#
#       Note that forms, patterns, and potentially other resources contain
#       images. This code only counts the top level images in a PDF document,
#       not images embedded in other resources.
@objc.callbackFor(Quartz.CGPDFOperatorTableSetCallback)
def myOperator_Do(s, info):
    # Check to see if this is an image or not.
    cs = Quartz.CGPDFScannerGetContentStream(s)

    # The Do operator takes a name. Pop the name off the
    # stack. If this fails then the argument to the
    # Do operator is not a name and is therefore invalid!
    res, name = Quartz.CGPDFScannerPopName(s, None)
    if not res:
        print("Couldn't pop name off stack!")
        return

    # Get the resource with type "XObject" and the name
    # obtained from the stack.
    xobject = Quartz.CGPDFContentStreamGetResource(cs, "XObject", name)
    if xobject is None:
        print("Couldn't get XObject with name %s" % (name,))
        return

    # An XObject must be a stream so obtain the value from the xobject
    # as if it were a stream. If this fails, the PDF is malformed.
    res, stream = Quartz.CGPDFObjectGetValue(
        xobject, Quartz.kCGPDFObjectTypeStream, None
    )
    if not res:
        print("XObject '%s' is not a stream" % (name,))
        return

    print(stream)

    # Streams consist of a dictionary and the data associated
    # with the stream. This code only cares about the dictionary.
    info_dict = Quartz.CGPDFStreamGetDictionary(stream)
    if info_dict is None:
        print("Couldn't obtain dictionary from stream %s!" % (name,))
        return

    # An XObject dict has a Subtype that indicates what kind it is.
    res, name = Quartz.CGPDFDictionaryGetName(info_dict, "Subtype", None)
    if not res:
        print("Couldn't get SubType of dictionary object!")
        return

    # This code is interested in the "Image" Subtype of an XObject.
    # Check whether this object has Subtype of "Image".
    if name != "Image":
        # The Subtype is not "Image" so this must be a form
        # or other type of XObject.
        return

    # This is an Image so figure out what variety of image it is.
    checkImageType(info_dict, info)


# This callback handles inline images. Inline images end with the
# "EI" operator.
@objc.callbackFor(Quartz.CGPDFOperatorTableSetCallback)
def myOperator_EI(s, info):
    print("EI")
    # When the scanner encounters the EI operator, it has a
    # stream corresponding to the image on the operand stack.
    # This code pops the stream off the stack in order to
    # examine it.
    res, stream = Quartz.CGPDFScannerPopStream(s, None)
    if not res:
        print("Couldn't create stream from inline image")
        return

    # Get the image dictionary from the stream.
    info_dict = Quartz.CGPDFStreamGetDictionary(stream)
    if info_dict is None:
        print("Couldn't get dict from inline image stream!")
        return

    # By definition the stream passed to EI is an image so
    # pass it to the code to check the type of image.
    checkImageType(info_dict, info)


def createMyOperatorTable():
    myTable = Quartz.CGPDFOperatorTableCreate()
    Quartz.CGPDFOperatorTableSetCallback(myTable, b"Do", myOperator_Do)
    Quartz.CGPDFOperatorTableSetCallback(myTable, b"EI", myOperator_EI)
    return myTable


def dumpPageStreams(url, outFile):
    # Create a CGPDFDocumentRef from the input PDF file.
    pdfDoc = Quartz.CGPDFDocumentCreateWithURL(url)
    if pdfDoc is None:
        print("Couldn't open PDF document!")
        return

    # Create the operator table with the needed callbacks.
    table = createMyOperatorTable()
    if table is None:
        print("Couldn't create operator table!")
        return

    # Initialize the count of the images.
    totalImages = 0

    # Obtain the total number of pages for the document.
    totPages = Quartz.CGPDFDocumentGetNumberOfPages(pdfDoc)

    # Loop over all the pages in the document, scanning the
    # content stream of each one.
    for i in range(1, totPages + 1):
        # Get the PDF page for this page in the document.
        p = Quartz.CGPDFDocumentGetPage(pdfDoc, i)

        # Create a reference to the content stream for this page.
        cs = Quartz.CGPDFContentStreamCreateWithPage(p)

        if cs is None:
            print("Couldn't create content stream for page #%d" % (i,))
            return

        # Initialize the counters of images for this page.
        myData = MyDataScan()

        # Create a scanner for this PDF document page.
        scanner = Quartz.CGPDFScannerCreate(cs, table, 0)
        if scanner is None:
            print("Couldn't create scanner for page #%d!" % (i,))
            return

        # CGPDFScannerScan causes Quartz to scan the content stream,
        # calling the callbacks in the table when the corresponding
        # operator is encountered. Once the content stream for the
        # page has been consumed or Quartz detects a malformed
        # content stream, CGPDFScannerScan returns.
        if not Quartz.CGPDFScannerScan(scanner):
            print("Scanner couldn't scan all of page #%d!" % (i,))

        # Print the results for this page.
        printPageResults(outFile, myData, i)

        # Update the total count of images with the count of the
        # images on this page.
        totalImages += (
            myData.numImagesWithColorThisPage
            + myData.numImageMasksThisPage
            + myData.numImagesMaskedWithMaskThisPage
            + myData.numImagesMaskedWithColorsThisPage
        )

        # Once the page has been scanned, release the
        # scanner for this page.
        Quartz.CGPDFScannerRelease(scanner)
        # Release the content stream for this page.
        Quartz.CGPDFContentStreamRelease(cs)
        # Done with this page; loop to next page.

    printDocResults(outFile, totPages, totalImages)


def main(args=None):
    if args is None:
        args = sys.argv

    if len(args) < 2:
        print("Usage: %s inputfile ... " % (args[0],))
        return 1

    for inputFileName in args[1:]:
        print("Beginning Document %r" % (inputFileName,))

        if not isinstance(inputFileName, bytes):
            inputFileName = inputFileName.encode("utf-8")

        inURL = Cocoa.CFURLCreateFromFileSystemRepresentation(
            None, inputFileName, len(inputFileName), False
        )
        if inURL is None:
            print("Couldn't create URL for input file!")
            return 1

        dumpPageStreams(inURL, sys.stdout)

    return 0


if __name__ == "__main__":
    sys.exit(main())



import math

import Cocoa
import LaunchServices
import objc
import Quartz


class ImageInfo(object):
    __slots__ = (
        "fRotation",
        "fScaleX",
        "fScaleY",
        "fTranslateX",
        "fTranslateY",
        "fImageRef",
        "fProperties",
        "fOrientation",
    )

    def __init__(self):
        self.fRotation = 0.0  # The rotation about the center of the image (degrees)
        self.fScaleX = 0.0  # The scaling of the image along it's X-axis
        self.fScaleY = 0.0  # The scaling of the image along it's Y-axis
        self.fTranslateX = 0.0  # Move the image along the X-axis
        self.fTranslateY = 0.0  # Move the image along the Y-axis
        self.fImageRef = None  # The image itself
        self.fProperties = None  # Image properties
        self.fOrientation = (
            None  # Affine transform that ensures the image displays correctly
        )


# Create a new image from a file at the given url
# Returns None if unsuccessful.
def IICreateImage(url):
    ii = None
    # Try to create an image source to the image passed to us
    imageSrc = Quartz.CGImageSourceCreateWithURL(url, None)
    if imageSrc is not None:
        # And if we can, try to obtain the first image available
        image = Quartz.CGImageSourceCreateImageAtIndex(imageSrc, 0, None)
        if image is not None:
            # and if we could, create the ImageInfo struct with default values
            ii = ImageInfo()
            ii.fRotation = 0.0
            ii.fScaleX = 1.0
            ii.fScaleY = 1.0
            ii.fTranslateX = 0.0
            ii.fTranslateY = 0.0
            # the ImageInfo struct owns this CGImageRef now, so no need for a retain.
            ii.fImageRef = image
            # the ImageInfo struct owns this CFDictionaryRef, so no need for a retain.
            ii.fProperties = Quartz.CGImageSourceCopyPropertiesAtIndex(
                imageSrc, 0, None
            )
            # Setup the orientation transformation matrix so that the image will
            # display with the proper orientation
            IIGetOrientationTransform(ii)

    return ii


# Transforms the context based on the orientation of the image.
# This ensures the image always appears correctly when drawn.
def IIGetOrientationTransform(image):
    w = Quartz.CGImageGetWidth(image.fImageRef)
    h = Quartz.CGImageGetHeight(image.fImageRef)
    if image.fProperties is not None:
        # The Orientations listed here are mirroed from CGImageProperties.h,
        # listed under the kCGImagePropertyOrientation key.
        orientation = IIGetImageOrientation(image)
        if orientation == 1:
            # 1 = 0th row is at the top, and 0th column is on the left.
            # Orientation Normal
            image.fOrientation = Quartz.CGAffineTransformMake(
                1.0, 0.0, 0.0, 1.0, 0.0, 0.0
            )

        elif orientation == 2:
            # 2 = 0th row is at the top, and 0th column is on the right.
            # Flip Horizontal
            image.fOrientation = Quartz.CGAffineTransformMake(
                -1.0, 0.0, 0.0, 1.0, w, 0.0
            )

        elif orientation == 3:
            # 3 = 0th row is at the bottom, and 0th column is on the right.
            # Rotate 180 degrees
            image.fOrientation = Quartz.CGAffineTransformMake(
                -1.0, 0.0, 0.0, -1.0, w, h
            )

        elif orientation == 4:
            # 4 = 0th row is at the bottom, and 0th column is on the left.
            # Flip Vertical
            image.fOrientation = Quartz.CGAffineTransformMake(1.0, 0.0, 0, -1.0, 0.0, h)

        elif orientation == 5:
            # 5 = 0th row is on the left, and 0th column is the top.
            # Rotate -90 degrees and Flip Vertical
            image.fOrientation = Quartz.CGAffineTransformMake(
                0.0, -1.0, -1.0, 0.0, h, w
            )

        elif orientation == 6:
            # 6 = 0th row is on the right, and 0th column is the top.
            # Rotate 90 degrees
            image.fOrientation = Quartz.CGAffineTransformMake(
                0.0, -1.0, 1.0, 0.0, 0.0, w
            )

        elif orientation == 7:
            # 7 = 0th row is on the right, and 0th column is the bottom.
            # Rotate 90 degrees and Flip Vertical
            image.fOrientation = Quartz.CGAffineTransformMake(
                0.0, 1.0, 1.0, 0.0, 0.0, 0.0
            )

        elif orientation == 8:
            # 8 = 0th row is on the left, and 0th column is the bottom.
            # Rotate -90 degrees
            image.fOrientation = Quartz.CGAffineTransformMake(
                0.0, 1.0, -1.0, 0.0, h, 0.0
            )


# Gets the orientation of the image from the properties dictionary if available
# If the kCGImagePropertyOrientation is not available or invalid,
# then 1, the default orientation, is returned.
def IIGetImageOrientation(image):
    result = 1
    if image.fProperties is not None:
        orientation = image.fProperties.get(Quartz.kCGImagePropertyOrientation)
        if orientation is not None:
            result = orientation

    return result


# Save the given image to a file at the given url.
# Returns true if successful, false otherwise.
def IISaveImage(image, url, width, height):
    result = False

    # If there is no image, no destination, or the width/height is 0, then fail early.
    assert (
        (image is not None) and (url is not None) and (width != 0.0) and (height != 0.0)
    )

    # Try to create a jpeg image destination at the url given to us
    imageDest = Quartz.CGImageDestinationCreateWithURL(
        url, LaunchServices.kUTTypeJPEG, 1, None
    )
    if imageDest is not None:
        # And if we can, then we can start building our final image.
        # We begin by creating a CGBitmapContext to host our desintation image.

        # Allocate enough space to hold our pixels
        imageData = objc.allocateBuffer(int(4 * width * height))

        # Create the bitmap context
        bitmapContext = Quartz.CGBitmapContextCreate(
            imageData,  # image data we just allocated...
            width,  # width
            height,  # height
            8,  # 8 bits per component
            4 * width,  # bytes per pixel times number of pixels wide
            Quartz.CGImageGetColorSpace(
                image.fImageRef
            ),  # use the same colorspace as the original image
            Quartz.kCGImageAlphaPremultipliedFirst,
        )  # use premultiplied alpha

        # Check that all that went well
        if bitmapContext is not None:
            # Now, we draw the image to the bitmap context
            IIDrawImageTransformed(
                image, bitmapContext, Quartz.CGRectMake(0.0, 0.0, width, height)
            )

            # We have now gotten our image data to the bitmap context, and correspondingly
            # into imageData. If we wanted to, we could look at any of the pixels of the image
            # and manipulate them in any way that we desire, but for this case, we're just
            # going to ask ImageIO to write this out to disk.

            # Obtain a CGImageRef from the bitmap context for ImageIO
            imageIOImage = Quartz.CGBitmapContextCreateImage(bitmapContext)

            # Check if we have additional properties from the original image
            if image.fProperties is not None:
                # If we do, then we want to inspect the orientation property.
                # If it exists and is not the default orientation, then we
                # want to replace that orientation in the destination file
                orientation = IIGetImageOrientation(image)
                if orientation != 1:
                    # If the orientation in the original image was not the default,
                    # then we need to replace that key in a duplicate of that dictionary
                    # and then pass that dictionary to ImageIO when adding the image.
                    prop = Cocoa.CFDictionaryCreateMutableCopy(
                        None, 0, image.fProperties
                    )
                    orientation = 1
                    prop[Quartz.kCGImagePropertyOrientation] = orientation

                    # And add the image with the new properties
                    Quartz.CGImageDestinationAddImage(imageDest, imageIOImage, prop)

                else:
                    # Otherwise, the image was already in the default orientation and we can
                    # just save it with the original properties.
                    Quartz.CGImageDestinationAddImage(
                        imageDest, imageIOImage, image.fProperties
                    )

            else:
                # If we don't, then just add the image without properties
                Quartz.CGImageDestinationAddImage(imageDest, imageIOImage, None)

            del bitmapContext

        # Finalize the image destination
        result = Quartz.CGImageDestinationFinalize(imageDest)
        del imageDest

    return result


# Applies the transformations specified in the ImageInfo struct without drawing the actual image
def IIApplyTransformation(image, context, bounds):
    if image is not None:
        # Whenever you do multiple CTM changes, you have to be very careful with
        # order.  Changing the order of your CTM changes changes the outcome of
        # the drawing operation. For example, if you scale a context by 2.0 along
        # the x-axis, and then translate the context by 10.0 along the x-axis,
        # then you will see your drawing will be in a different position than if
        # you had done the operations in the opposite order.
        #
        # Our intent with this operation is that we want to change the location
        # from which we start drawing (translation), then rotate our axies so
        # that our image appears at an angle (rotation), and finally
        # scale our axies so that our image has a different size (scale).
        # Changing the order of operations will markedly change the results.
        IITranslateContext(image, context)
        IIRotateContext(image, context, bounds)
        IIScaleContext(image, context, bounds)


# Draw the image to the given context centered inside the given bounds
def IIDrawImage(image, context, bounds):
    imageRect = Cocoa.NSRect()
    if image is not None and context is not None:
        # Setup the image rect so that the image fills it's natural boudaries
        # in the base coordinate system.
        imageRect.origin.x = 0.0
        imageRect.origin.y = 0.0
        imageRect.size.width = Quartz.CGImageGetWidth(image.fImageRef)
        imageRect.size.height = Quartz.CGImageGetHeight(image.fImageRef)

        # Obtain the orientation matrix for this image
        ctm = image.fOrientation

        # Before we can apply the orientation matrix, we need to translate the
        # coordinate system so the center of the rectangle matces the center of
        # the image.
        if image.fProperties is None or IIGetImageOrientation(image) < 5:
            # For orientations 1-4, the images are unrotated, so the width and
            # height of the base image can be used as the width and height of
            # the coordinate translation calculation.
            Quartz.CGContextTranslateCTM(
                context,
                math.floor((bounds.size.width - imageRect.size.width) / 2.0),
                math.floor((bounds.size.height - imageRect.size.height) / 2.0),
            )

        else:
            # For orientations 5-8, the images are rotated 90 or -90 degrees,
            # so we need to use the image width in place of the height and
            # vice versa.
            Quartz.CGContextTranslateCTM(
                context,
                math.floor((bounds.size.width - imageRect.size.height) / 2.0),
                math.floor((bounds.size.height - imageRect.size.width) / 2.0),
            )

        # Finally, orient the context so that the image draws naturally.
        Quartz.CGContextConcatCTM(context, ctm)

        # And draw the image.
        Quartz.CGContextDrawImage(context, imageRect, image.fImageRef)


# Rotates the context around the center point of the given bounds
def IIRotateContext(image, context, bounds):
    # First we translate the context such that the 0,0 location is at the center of the bounds
    Quartz.CGContextTranslateCTM(
        context, bounds.size.width / 2.0, bounds.size.height / 2.0
    )

    # Then we rotate the context, converting our angle from degrees to radians
    Quartz.CGContextRotateCTM(context, image.fRotation * math.pi / 180.0)

    # Finally we have to restore the center position
    Quartz.CGContextTranslateCTM(
        context, -bounds.size.width / 2.0, -bounds.size.height / 2.0
    )


# Scale the context around the center point of the given bounds
def IIScaleContext(image, context, bounds):
    # First we translate the context such that the 0,0 location is at the center of the bounds
    Quartz.CGContextTranslateCTM(
        context, bounds.size.width / 2.0, bounds.size.height / 2.0
    )

    # Next we scale the context to the size that we want
    Quartz.CGContextScaleCTM(context, image.fScaleX, image.fScaleY)

    # Finally we have to restore the center position
    Quartz.CGContextTranslateCTM(
        context, -bounds.size.width / 2.0, -bounds.size.height / 2.0
    )


# Translate the context
def IITranslateContext(image, context):
    # Translation is easy, just translate.
    Quartz.CGContextTranslateCTM(context, image.fTranslateX, image.fTranslateY)


# Draw the image to the given context centered inside the given bounds with
# the transformation info. The CTM of the context is unchanged after this call
def IIDrawImageTransformed(image, context, bounds):
    # We save the current graphics state so as to not disrupt it for the caller.
    Quartz.CGContextSaveGState(context)

    # Apply the transformation
    IIApplyTransformation(image, context, bounds)

    # Draw the image centered in the context
    IIDrawImage(image, context, bounds)

    # Restore our original graphics state.
    Quartz.CGContextRestoreGState(context)


# Release the ImageInfo struct and other associated data
# you should not refer to the reference after this call
# This function is None safe.
def IIRelease(image):
    pass



import Cocoa
import objc
import Quartz
import QuartzTextDrawing
import Utilities
from objc import super


def getTextString():
    # These unicode values are the characters: Q, u, a, r, t, z,
    # eighthnote, floral heart, black chess queen, and two CJK characters.
    # Note: Create an NSString, because we'll use NSString-specific API's, otherwise
    # we could just have used a python unicode object
    return Cocoa.NSString.stringWithString_(
        "\u0051\u0075\u0061\u0072\u0074\u007A\u266A\u2766\u265B\u3042\u304E"
    )


doPointDrawing = 1


def drawNSStringWithAttributes():
    textString = getTextString()
    if doPointDrawing:
        context = Cocoa.NSGraphicsContext.currentContext().graphicsPort()

    # Text Line 1. Draw with default attributes.
    p = Cocoa.NSMakePoint(20.0, 400.0)

    # Draw text with default text attributes. The point supplied is
    # not the text baseline but rather the lower-left corner of the box
    # which bounds the text.
    textString.drawAtPoint_withAttributes_(p, None)

    if doPointDrawing:
        Utilities.drawPoint(context, p)

    # Text Line 2. Draw with a specific font and color.

    # Position the text 50 units below the previous text.
    p.y -= 50

    # Set attributes to use when drawing the string.
    stringAttributes = {
        # Use the font with the PostScript name "Times-Roman" at 40 point.
        Cocoa.NSFontAttributeName: Cocoa.NSFont.fontWithName_size_("Times-Roman", 40),
        # Set the color attribute to an opaque red.
        Cocoa.NSForegroundColorAttributeName: Cocoa.NSColor.colorWithCalibratedRed_green_blue_alpha_(  # noqa: B950
            0.663, 0, 0.031, 1.0
        ),
    }

    # Draw the text.
    textString.drawAtPoint_withAttributes_(p, stringAttributes)

    if doPointDrawing:
        Utilities.drawPoint(context, p)

    # Text Line 3. Draw stroked text.

    # Position the text 50 units below the previous text.
    p.y -= 50

    # Panther and later support stroke attributes. A positive value
    # of the stroke width attribute produces text that is stroked rather
    # than filled.
    stringAttributes[Cocoa.NSStrokeWidthAttributeName] = 3.0
    textString.drawAtPoint_withAttributes_(p, stringAttributes)

    if doPointDrawing:
        Utilities.drawPoint(context, p)

    # Text Line 4. Draw with fill and stroke.

    p.y -= 50

    # Panther and later support stroke attributes. A negative value
    # of the stroke width attribute results in text that is both filled
    # and stroked.
    stringAttributes[Cocoa.NSStrokeWidthAttributeName] = -3.0
    # Set the stroke color attribute to black.
    stringAttributes[
        Cocoa.NSStrokeColorAttributeName
    ] = Cocoa.NSColor.colorWithCalibratedRed_green_blue_alpha_(0, 0, 0, 1.0)

    textString.drawAtPoint_withAttributes_(p, stringAttributes)

    if doPointDrawing:
        Utilities.drawPoint(context, p)

    # Text Line 5. Draw at baseline.
    # Tiger and later support the drawWithRect method which allows
    # string text drawing from a point on the text baseline.
    p.y -= 50
    rect = Cocoa.NSRect(origin=p, size=Cocoa.NSSize(0, 0))
    textString.drawWithRect_options_attributes_(
        rect, Cocoa.NSStringDrawingDisableScreenFontSubstitution, stringAttributes
    )

    if doPointDrawing:
        Utilities.drawPoint(context, p)


_myLayout = None
_textStorage = None
_myTextRange = None


def drawWithNSLayout():
    global _myLayout, _textStorage, _myTextRange

    if _myLayout is None:
        # Initialize the text storage with the string to draw.
        _textStorage = Cocoa.NSTextStorage.alloc().initWithString_(getTextString())
        # Initialize the layout manager to use with the text storage.
        _myLayout = Cocoa.NSLayoutManager.alloc().init()
        # Allocate and initialize a text container object.
        textContainer = Cocoa.NSTextContainer.alloc().init()
        # Add the text container to the layout.
        _myLayout.addTextContainer_(textContainer)
        # Release the text container since the layout retains it and
        # this code no longer needs it.
        del textContainer
        # Add the layout to the text storage.
        _textStorage.addLayoutManager_(_myLayout)

        # Set attributes to use when drawing the string.
        stringAttributes = {
            # Use the font with the PostScript name "Times-Roman" at 40 point.
            Cocoa.NSFontAttributeName: Cocoa.NSFont.fontWithName_size_(
                "Times-Roman", 40
            ),
            # Set the text color attribute to an opaque red.
            Cocoa.NSForegroundColorAttributeName: Cocoa.NSColor.colorWithCalibratedRed_green_blue_alpha_(  # noqa: B950
                0.663, 0, 0.031, 1.0
            ),
        }

        # Create the range of text for the entire length of text
        # in the textStorage object.
        _myTextRange = Cocoa.NSMakeRange(0, _textStorage.length())
        # Set the attributes on the entire range of text.
        _textStorage.setAttributes_range_(stringAttributes, _myTextRange)

    # Set the point for drawing the layout.
    p = Cocoa.NSMakePoint(20.0, 400.0)

    # Draw the text range at the point.
    _myLayout.drawGlyphsForGlyphRange_atPoint_(_myTextRange, p)

    if doPointDrawing:
        context = Cocoa.NSGraphicsContext.currentContext().graphicsPort()
        Utilities.drawPoint(context, p)


# The interface to the NSLayoutManager subclass.
class MyNSLayoutManager(Cocoa.NSLayoutManager):
    # The extra instance variables for this subclass.
    _textMode = objc.ivar()
    _fColor = objc.ivar()
    _sColor = objc.ivar()
    _yStartPosition = objc.ivar()
    _lineWidth = objc.ivar()
    _clippingDrawProc = objc.ivar()
    _clippingInfo = objc.ivar()

    # Public methods to set the special attributes
    # of the MyNSLayoutManager instance.
    def setTextMode_(self, textMode):
        self._textMode = textMode

    def setFillColor_(self, color):
        self._fColor = color

    def setStrokeColor_(self, color):
        self._sColor = color

    def setTextLineWidth_(self, width):
        self._lineWidth = width

    def setClippingDrawProc_withInfo_(self, clippingDrawProc, info):
        self._clippingDrawProc = clippingDrawProc
        self._clippingInfo = info

    def init(self):
        self = super(MyNSLayoutManager, self).init()
        if self is None:
            return None

        # Initialize the custom instance variables.
        self._textMode = Quartz.kCGTextFill
        self._fColor = None
        self._sColor = None
        self._yStartPosition = 0
        self._lineWidth = 1
        self._clippingDrawProc = None
        self._clippingInfo = None
        return self

    # This code overrides this method to record the y coordinate
    # to use as the True baseline for the text drawing.
    def drawGlyphsForGlyphRange_atPoint_(self, glyphsToShow, origin):
        self._yStartPosition = origin.y
        super(MyNSLayoutManager, self).drawGlyphsForGlyphRange_atPoint_(
            glyphsToShow, origin
        )

    # This is the rendering method of NSLayoutManager that the
    # code overrides to perform its custom rendering.
    def showPackedGlyphs_length_glyphRange_atPoint_font_color_printAdjustment_(
        self, glyphs, glyphLen, glyphRange, point, font, color, printingAdjustment
    ):

        # Obtain the destination drawing context.
        context = Cocoa.NSGraphicsContext.currentContext().graphicsPort()

        # Adjust start position y value based on the adjusted y coordinate.
        # This ensures the text baseline is at the starting position
        # passed to drawGlyphsForGlyphRange. This technique won't work
        # for super, subscripts, or underlines but that's OK for this example.
        point.y = self._yStartPosition

        # The Quartz graphics state should be preserved by showPackedGlyphs.
        Quartz.CGContextSaveGState(context)

        # Set the desired text drawing mode.
        Quartz.CGContextSetTextDrawingMode(context, self._textMode)

        # Set the fill color if needed.
        if (
            self._textMode == Quartz.kCGTextFill
            or self.textMode == Quartz.kCGTextFillStroke
            or self._textMode == Quartz.kCGTextFillClip
            or self._textMode == Quartz.kCGTextFillStrokeClip
        ):
            if self._fColor is not None:
                Quartz.CGContextSetFillColorWithColor(context, self._fColor)

        # Set the  line width and the stroke color if needed.
        if (
            self._textMode == Quartz.kCGTextStroke
            or self._textMode == Quartz.kCGTextFillStroke
            or self._textMode == Quartz.kCGTextStrokeClip
            or self._textMode == Quartz.kCGTextFillStrokeClip
        ):
            Quartz.CGContextSetLineWidth(context, self._lineWidth)
            if self._sColor is not None:
                Quartz.CGContextSetStrokeColorWithColor(context, self._sColor)

        # Now draw the text. Check whether to adjust for printing widths
        # and if needed adjust extra character spacing accordingly.
        if printingAdjustment.width != 0.0:
            # If printingAdjustment width is non-zero then the text
            # needs to be adjusted. printingAdjustment is the per character
            # adjustment required for this piece of text. Because
            # the Quartz text character spacing set is transformed by
            # the text matrix, this code needs to factor out that effect
            # prior to setting it. Cocoa sets the text matrix to account
            # for the point size of the font so we factor that out of the
            # per character width supplied here.
            charAdjust = printingAdjustment.width / font.pointSize()
            Quartz.CGContextSetCharacterSpacing(context, charAdjust)
        else:
            Quartz.CGContextSetCharacterSpacing(context, 0.0)

        # Draw the glyphs. The total number of glyphs is the length
        # of the glyphs string passed to showPackedGlyphs, divided by 2
        # since there are two bytes per glyph.
        Quartz.CGContextShowGlyphsAtPoint(
            context, point.x, point.y, glyphs, glyphLen / 2
        )

        # If the text drawing mode requires clipping and there is
        # a custom clipping proc, call it. This allows drawing through
        # clipped text before the graphics state is restored.
        if (
            self._textMode == Quartz.kCGTextClip
            or self._textMode == Quartz.kCGTextFillClip
            or self._textMode == Quartz.kCGTextStrokeClip
            or self._textMode == Quartz.kCGTextFillStrokeClip
        ) and self._clippingDrawProc is not None:

            self._clippingDrawProc(context, point.x, point.y, self._clippingInfo)

        Quartz.CGContextRestoreGState(context)


def MyClipProc(c, x, y, info):
    Quartz.CGContextTranslateCTM(c, x, y)
    Quartz.CGContextSetStrokeColorWithColor(c, Utilities.getRGBOpaqueBlackColor())
    # Draw a grid of lines through the clip.
    QuartzTextDrawing.drawGridLines(c)


_myLayout2 = None
_textStorage2 = None
_myTextRange2 = None


def drawWithCustomNSLayout():
    global _myLayout2, _textStorage2, _myTextRange2

    if _myLayout2 is None:
        textContainer = Cocoa.NSTextContainer.alloc().init()

        _textStorage2 = Cocoa.NSTextStorage.alloc().initWithString_(getTextString())
        # Create an instance of the MyNSLayoutManager subclass of NSLayoutManager.
        _myLayout2 = MyNSLayoutManager.alloc().init()
        _myLayout2.addTextContainer_(textContainer)
        # The layout retains the text container so this code can release it.
        del textContainer
        _textStorage2.addLayoutManager_(_myLayout2)

        # Set attributes to use when drawing the string.
        stringAttributes = {
            # Use the font with the PostScript name "Times-Roman" at 40 point.
            Cocoa.NSFontAttributeName: Cocoa.NSFont.fontWithName_size_(
                "Times-Roman", 40
            )
        }

        # Create the range.
        _myTextRange2 = Cocoa.NSMakeRange(0, _textStorage2.length())
        # Set the attributes on the entire range of text.
        _textStorage2.setAttributes_range_(stringAttributes, _myTextRange2)

    p = Cocoa.NSMakePoint(20.0, 400.0)

    # Set the custom attributes of the layout subclass so that
    # the text will be filled with black.
    _myLayout2.setTextMode_(Quartz.kCGTextFill)
    _myLayout2.setFillColor_(Utilities.getRGBOpaqueBlackColor())

    # Draw text line 1.
    _myLayout2.drawGlyphsForGlyphRange_atPoint_(_myTextRange2, p)

    if doPointDrawing:
        context = Cocoa.NSGraphicsContext.currentContext().graphicsPort()
        Utilities.drawPoint(context, p)

    # Set the custom attributes of the layout subclass so that
    # the text will be stroked with black.
    _myLayout2.setTextMode_(Quartz.kCGTextStroke)
    _myLayout2.setStrokeColor_(Utilities.getRGBOpaqueBlackColor())
    _myLayout2.setTextLineWidth_(2)

    # Draw text line 2.
    p.y -= 50
    _myLayout2.drawGlyphsForGlyphRange_atPoint_(_myTextRange2, p)

    if doPointDrawing:
        Utilities.drawPoint(context, p)

    p.y -= 50

    # Set the custom attributes of the layout subclass so that
    # the text will be filled and stroked and the fill color
    # will be red. Since the stroke color hasn't changed it
    # will be stroked with black.
    _myLayout2.setTextMode_(Quartz.kCGTextFillStroke)
    _myLayout2.setFillColor_(Utilities.getRGBOpaqueRedColor())
    # Draw text line 3.
    _myLayout2.drawGlyphsForGlyphRange_atPoint_(_myTextRange2, p)

    if doPointDrawing:
        Utilities.drawPoint(context, p)

    p.y -= 50

    # Set the custom attributes of the layout subclass so that
    # the text will be filled, stroked, then clipped.
    _myLayout2.setTextMode_(Quartz.kCGTextFillStrokeClip)

    # Set the clipping proc to MyClipProc which requires
    # no info data.
    _myLayout2.setClippingDrawProc_withInfo_(MyClipProc, None)

    # Draw text line 4.
    _myLayout2.drawGlyphsForGlyphRange_atPoint_(_myTextRange2, p)

    if doPointDrawing:
        Utilities.drawPoint(context, p)

    # Set the clipping proc to None for future drawing.
    _myLayout2.setClippingDrawProc_withInfo_(None, None)



import math

import CGImageUtils
import Cocoa
import LaunchServices
import objc
import Quartz


class Controller(Cocoa.NSObject):
    imageView = objc.IBOutlet()
    scaleYView = objc.IBOutlet()
    textScaleYView = objc.IBOutlet()

    _rotation = objc.ivar.float()
    _scaleX = objc.ivar.float()
    _scaleY = objc.ivar.float()
    _translateX = objc.ivar.float()
    _translateY = objc.ivar.float()
    _preserveAspectRatio = objc.ivar.bool()

    openImageIOSupportedTypes = objc.ivar()

    def awakeFromNib(self):
        self.openImageIOSupportedTypes = None
        # Ask CFBundle for the location of our demo image
        url = Cocoa.CFBundleCopyResourceURL(
            Cocoa.CFBundleGetMainBundle(), "demo", "png", None
        )
        if url is not None:
            # And if available, load it
            self.imageView.setImage_(CGImageUtils.IICreateImage(url))

        self.imageView.window().center()
        self.setRotation_(0.0)
        self.setScaleX_(1.0)
        self.setScaleY_(1.0)
        self.setTranslateX_(0.0)
        self.setTranslateY_(0.0)
        self.setPreserveAspectRatio_(False)

    @objc.IBAction
    def changeScaleX_(self, sender):
        self.setScaleX_(self._scaleX + sender.floatValue())
        sender.setFloatValue_(0.0)

    @objc.IBAction
    def changeScaleY_(self, sender):
        self.setScaleY_(self._scaleY + sender.floatValue())
        sender.setFloatValue_(0.0)

    @objc.IBAction
    def changeTranslateX_(self, sender):
        self.setTranslateX_(self._translateX + sender.floatValue())
        sender.setFloatValue_(0.0)

    @objc.IBAction
    def changeTranslateY_(self, sender):
        self.setTranslateY_(self._translateY + sender.floatValue())
        sender.setFloatValue_(0.0)

    @objc.IBAction
    def reset_(self, sender):
        self.setRotation_(0.0)
        self.setScaleX_(1.0)
        self.setScaleY_(1.0)
        self.setTranslateX_(0.0)
        self.setTranslateY_(0.0)

        self.imageView.setNeedsDisplay_(True)

    def extensionsForUTI_(self, uti):
        """
        Returns an array with the extensions that match the given
        Uniform Type Identifier (UTI).
        """
        # If anything goes wrong, we'll return None, otherwise this will be the array
        # of extensions for this image type.
        extensions = None
        # Only get extensions for UTIs that are images (i.e. conforms to
        # public.image aka kUTTypeImage) This excludes PDF support that ImageIO
        # advertises, but won't actually use.
        if LaunchServices.UTTypeConformsTo(uti, LaunchServices.kUTTypeImage):
            # Copy the decleration for the UTI (if it exists)
            decleration = LaunchServices.UTTypeCopyDeclaration(uti)
            if decleration is not None:
                # Grab the tags for this UTI, which includes extensions, OSTypes and MIME types.
                tags = Cocoa.CFDictionaryGetValue(
                    decleration, LaunchServices.kUTTypeTagSpecificationKey
                )
                if tags is not None:
                    # We are interested specifically in the extensions that this UTI uses
                    filenameExtensions = tags.get(
                        LaunchServices.kUTTagClassFilenameExtension
                    )
                    if filenameExtensions is not None:
                        # It is valid for a UTI to export either an Array
                        # (of Strings) representing multiple tags, or a String
                        # representing a single tag.
                        type_id = Cocoa.CFGetTypeID(filenameExtensions)
                        if type_id == Cocoa.CFStringGetTypeID():
                            # If a string was exported, then wrap it up in an array.
                            extensions = Cocoa.NSArray.arrayWithObject_(
                                filenameExtensions
                            )
                        elif type_id == Cocoa.CFArrayGetTypeID():
                            # If an array was exported, then just return that array.
                            extensions = filenameExtensions.copy()

        return extensions

    # On Tiger NSOpenPanel only understands extensions, not UTIs, so we have to
    # obtain a list of extentions from the UTIs that Image IO tells us it can
    # handle.
    def createOpenTypesArray(self):
        if self.openImageIOSupportedTypes is None:
            imageIOUTIs = Quartz.CGImageSourceCopyTypeIdentifiers()
            count = len(imageIOUTIs)
            self.openImageIOSupportedTypes = Cocoa.NSMutableArray.alloc().initWithCapacity_(
                count
            )
            for i in range(count):
                self.openImageIOSupportedTypes.addObjectsFromArray_(
                    self.extensionsForUTI_(imageIOUTIs[i])
                )

    @objc.IBAction
    def openDocument_(self, sender):
        panel = Cocoa.NSOpenPanel.openPanel()
        panel.setAllowsMultipleSelection_(False)
        panel.setResolvesAliases_(True)
        panel.setTreatsFilePackagesAsDirectories_(True)

        self.createOpenTypesArray()

        panel.beginSheetForDirectory_file_types_modalForWindow_modalDelegate_didEndSelector_contextInfo_(  # noqa: B950
            None,
            None,
            self.openImageIOSupportedTypes,
            self.imageView.window(),
            self,
            "openImageDidEnd:returnCode:contextInfo:",
            None,
        )

    @objc.signature(b"v@:@i^v")
    def openImageDidEnd_returnCode_contextInfo_(self, panel, returnCode, contextInfo_):
        if returnCode == Cocoa.NSOKButton:
            if len(panel.filenames()) > 0:
                image = CGImageUtils.IICreateImage(
                    Cocoa.NSURL.fileURLWithPath_(panel.filenames()[0])
                )
                if image is not None:
                    # Ownership is transferred to the CGImageView.
                    self.imageView.setImage_(image)

    @objc.IBAction
    def saveDocumentAs_(self, sender):
        panel = Cocoa.NSSavePanel.savePanel()
        panel.setCanSelectHiddenExtension_(True)
        panel.setRequiredFileType_("jpeg")
        panel.setAllowsOtherFileTypes_(False)
        panel.setTreatsFilePackagesAsDirectories_(True)

        panel.beginSheetForDirectory_file_modalForWindow_modalDelegate_didEndSelector_contextInfo_(  # noqa: B950
            None,
            "untitled image",
            self.imageView.window(),
            self,
            "saveImageDidEnd:returnCode:contextInfo:",
            None,
        )

    @objc.signature(b"v@:@i^v")
    def saveImageDidEnd_returnCode_contextInfo_(self, panel, returnCode, contextInfo):
        if returnCode == Cocoa.NSOKButton:
            frame = self.imageView.frame()
            CGImageUtils.IISaveImage(
                self.imageView.image(),
                panel.URL(),
                math.ceil(frame.size.width),
                math.ceil(frame.size.height),
            )

    def setRotation_(self, r):
        r = r % 360.0
        if r < 0:
            r += 360.0

        self._rotation = r
        self.imageView.image().fRotation = 360.0 - r  # XXX
        self.imageView.setNeedsDisplay_(True)

    def setScaleX_(self, x):
        self._scaleX = x
        self.imageView.image().fScaleX = self._scaleX
        if self._preserveAspectRatio:
            self.imageView.image().fScaleY = self._scaleX

        self.imageView.setNeedsDisplay_(True)

    def setScaleY_(self, y):
        self._scaleY = y
        if not self._preserveAspectRatio:
            self.imageView.image().fScaleY = self._scaleY
            self.imageView.setNeedsDisplay_(True)

    def setPreserveAspectRatio_(self, preserve):
        self._preserveAspectRatio = preserve
        self.imageView.image().fScaleX = self._scaleX
        if self._preserveAspectRatio:
            self.imageView.image().fScaleY = self._scaleX

        else:
            self.imageView.image().fScaleY = self._scaleY

        self.scaleYView.setEnabled_(not self._preserveAspectRatio)
        self.textScaleYView.setEnabled_(not self._preserveAspectRatio)
        self.imageView.setNeedsDisplay_(True)

    def setTranslateX_(self, x):
        self._translateX = x
        self.imageView.image().fTranslateX = self._translateX
        self.imageView.setNeedsDisplay_(True)

    def setTranslateY_(self, y):
        self._translateY = y
        self.imageView.image().fTranslateY = self._translateY
        self.imageView.setNeedsDisplay_(True)

    def rotation(self):
        return self._rotation

    def scaleX(self):
        return self._scaleX

    def scaleY(self):
        return self._scaleY

    def preserveAspectRatio(self):
        return self._preserveAspectRatio

    def translateX(self):
        return self._translateX

    def translateY(self):
        return self._translateY



import AppDrawing
import Cocoa
import objc


def getURLToExport(suffix):
    savePanel = Cocoa.NSSavePanel.savePanel()

    initialFileName = "Quartz2DBasics.%s" % (suffix,)

    if (
        savePanel.runModalForDirectory_file_(None, initialFileName)
        == Cocoa.NSFileHandlingPanelOKButton
    ):
        return savePanel.URL()

    return None


class MyAppController(Cocoa.NSObject):
    theView = objc.IBOutlet()

    @objc.IBAction
    def print_(self, sender):
        self.theView.print_(sender)

    @objc.IBAction
    def exportAsPNG_(self, sender):
        url = getURLToExport("png")
        if url:
            AppDrawing.myExportCGDrawingAsPNG(
                url, self.theView.currentPrintableCommand()
            )

    @objc.IBAction
    def exportAsPDF_(self, sender):
        url = getURLToExport("pdf")
        if url:
            AppDrawing.myCreatePDFDocument(url, self.theView.currentPrintableCommand())



import math

import Quartz
import Utilities


def doRotatedEllipses(context):
    totreps = 144
    tint = 1.0
    tintIncrement = 1.0 / totreps
    # Create a new transform consisting of a 45 degrees rotation.
    theTransform = Quartz.CGAffineTransformMakeRotation(math.pi / 4)
    # Apply a scale to the transform just created.
    theTransform = Quartz.CGAffineTransformScale(theTransform, 1, 2)
    # Place the first ellipse at a good location.
    Quartz.CGContextTranslateCTM(context, 100.0, 100.0)

    for _ in range(totreps):
        # Make a snapshot the coordinate system.
        Quartz.CGContextSaveGState(context)
        # Set up the coordinate system for the rotated ellipse.
        Quartz.CGContextConcatCTM(context, theTransform)
        Quartz.CGContextBeginPath(context)
        Quartz.CGContextAddArc(context, 0.0, 0.0, 45.0, 0.0, 2 * math.pi, 0)
        # Set the fill color for this instance of the ellipse.
        Quartz.CGContextSetRGBFillColor(context, tint, 0.0, 0.0, 1.0)
        Quartz.CGContextDrawPath(context, Quartz.kCGPathFill)
        # Restore the coordinate system to that of the snapshot.
        Quartz.CGContextRestoreGState(context)
        # Compute the next tint color.
        tint -= tintIncrement
        # Move over by 1 unit in x for the next ellipse.
        Quartz.CGContextTranslateCTM(context, 1.0, 0.0)


def drawSkewedCoordinateSystem(context):
    # alpha is 22.5 degrees and beta is 15 degrees.
    alpha = math.pi / 8
    beta = math.pi / 12
    # Create a rectangle that is 72 units on a side
    # with its origin at (0,0).
    r = Quartz.CGRectMake(0, 0, 72, 72)

    Quartz.CGContextTranslateCTM(context, 144, 144)
    # Draw the coordinate axes untransformed.
    Utilities.drawCoordinateAxes(context)
    # Fill the rectangle.
    Quartz.CGContextFillRect(context, r)

    # Create an affine transform that skews the coordinate system,
    # skewing the x-axis by alpha radians and the y-axis by beta radians.
    skew = Quartz.CGAffineTransformMake(1, math.tan(alpha), math.tan(beta), 1, 0, 0)
    # Apply that transform to the context coordinate system.
    Quartz.CGContextConcatCTM(context, skew)

    # Set the fill and stroke color to a dark blue.
    Quartz.CGContextSetRGBStrokeColor(context, 0.11, 0.208, 0.451, 1)
    Quartz.CGContextSetRGBFillColor(context, 0.11, 0.208, 0.451, 1)

    # Draw the coordinate axes again, now transformed.
    Utilities.drawCoordinateAxes(context)
    # Set the fill color again but with a partially transparent alpha.
    Quartz.CGContextSetRGBFillColor(context, 0.11, 0.208, 0.451, 0.7)
    # Fill the rectangle in the transformed coordinate system.
    Quartz.CGContextFillRect(context, r)



import Cocoa
import objc
import Quartz
from objc import super

_hazeRemovalKernel = None


class MyHazeFilter(Quartz.CIFilter):
    inputImage = objc.ivar()
    inputColor = objc.ivar()
    inputDistance = objc.ivar()
    inputSlope = objc.ivar()

    @classmethod
    def initialize(cls):
        Quartz.CIFilter.registerFilterName_constructor_classAttributes_(
            "MyHazeRemover",
            cls,
            {
                Quartz.kCIAttributeFilterDisplayName: "Haze Remover",
                Quartz.kCIAttributeFilterCategories: [
                    Quartz.kCICategoryColorAdjustment,
                    Quartz.kCICategoryVideo,
                    Quartz.kCICategoryStillImage,
                    Quartz.kCICategoryInterlaced,
                    Quartz.kCICategoryNonSquarePixels,
                ],
                "inputDistance": {
                    Quartz.kCIAttributeMin: 0.0,
                    Quartz.kCIAttributeMax: 1.0,
                    Quartz.kCIAttributeSliderMin: 0.0,
                    Quartz.kCIAttributeSliderMax: 0.7,
                    Quartz.kCIAttributeDefault: 0.2,
                    Quartz.kCIAttributeIdentity: 0.0,
                    Quartz.kCIAttributeType: Quartz.kCIAttributeTypeScalar,
                },
                "inputSlope": {
                    Quartz.kCIAttributeSliderMin: -0.01,
                    Quartz.kCIAttributeSliderMax: 0.01,
                    Quartz.kCIAttributeDefault: 0.0,
                    Quartz.kCIAttributeIdentity: 0.0,
                    Quartz.kCIAttributeType: Quartz.kCIAttributeTypeScalar,
                },
                "inputColor": {
                    Quartz.kCIAttributeDefault: Quartz.CIColor.colorWithRed_green_blue_alpha_(
                        1.0, 1.0, 1.0, 1.0
                    )
                },
            },
        )

    @classmethod
    def filterWithName_(cls, name):
        return cls.alloc().init()

    def init(self):
        global _hazeRemovalKernel

        if _hazeRemovalKernel is None:
            bundle = Cocoa.NSBundle.bundleForClass_(type(self))
            with open(
                bundle.pathForResource_ofType_("MyHazeRemoval", "cikernel"), "r"
            ) as fp:
                code = fp.read()
            kernels = Quartz.CIKernel.kernelsWithString_(code)
            _hazeRemovalKernel = kernels[0]

        return super(MyHazeFilter, self).init()

    def outputImage(self):
        src = Quartz.CISampler.samplerWithImage_(self.inputImage)

        return self.apply_arguments_options_(
            _hazeRemovalKernel,
            (src, self.inputColor, self.inputDistance, self.inputSlope),
            {"definition": src.definition()},
        )
